@@ -156,7 +156,7 @@ void reconnect()
 			oer_debug(OER_DEBUG_INFO, "reconnect->oer+MySQL exiting\n");
 #endif
 			close(mystate->sockfd);
-			mysql_close(&mystate->mysqldb.mysqldbconn);
+			mysql_close(&mystate->mysqldb->mysqldbconn);
 			exit(EXIT_SUCCESS);
 		}
 #ifdef DEBUG
@@ -423,8 +423,8 @@ int handleserverdata()
 	/* here we log all the server traffic, prefix with timestamp */
 	if((index(mystate->flags, (int)'l') != NULL) && (strstr(mystate->state, "+ro") == NULL)) {
 		/* it doesn't matter if logging fails */
-		mysql_real_escape_string(&mystate->mysqldb.mysqldbconn, mysql_safe_str1, stringbuffer, strlen(stringbuffer));
-		oer_doquery("handleserverdata", OER_DEBUG_NOISE, "INSERT INTO oer_raw VALUES (%lu, '%s', '%s')", mystate->now, mystate->ident, mysql_safe_str1);
+		mysql_real_escape_string(&mystate->mysqldb->mysqldbconn, mysql_safe_str1, stringbuffer, strlen(stringbuffer));
+		oer_doquery(mystate->mysqldb, "handleserverdata", OER_DEBUG_NOISE, "INSERT INTO oer_raw VALUES (%lu, '%s', '%s')", mystate->now, mystate->ident, mysql_safe_str1);
 	}
 	if((ptr = (char *) strstr(stringbuffer, "PING :")) != NULL) {
 		snprintf(pong, STRINGLEN, "PONG :%s\n", ptr + 6);
