@@ -266,7 +266,7 @@ enum OER_FLAGS_TYPE {
         OER_FLAGS_TYPE_USER
 };
 
-#define OER_FLAGS_GLOBAL "dfglmnopqs"
+#define OER_FLAGS_GLOBAL "dfglmnopqsS"
 #define OER_FLAGS_CHANNEL "!aAbcDefFGklLmMnNoOpPqrRsSuUtTvVwxX"
 #define OER_FLAGS_ADMIN "dn"
 #define OER_FLAGS_USER "!adfmnovrsx"
@@ -357,7 +357,7 @@ enum OER_FLAGS_TYPE {
 #define OER_PUBMSGS 20
 #define OER_NICKCHANGES 20
 #define OER_SERVER_IS_STONED_FOR 300
-#define OER_LINENOISE_INTERVAL_MIN 1
+#define OER_LINENOISE_INTERVAL_MIN 0
 #define OER_MASSMODE_LIMIT 3
 #define OER_LAST 3
 #define OER_LAST_MAX 10
@@ -365,7 +365,7 @@ enum OER_FLAGS_TYPE {
 #define OER_SEEN 3
 #define OER_SEEN_MAX 10
 #define OER_TOPICS 20
-#define OER_TOPICLEN 160
+#define OER_TOPICLEN 250
 #define TOPIC_SEPARATOR " | "
 #define CONNECTION_STATE_PREFIXES "^~+=-"
 #define OER_PERMBANS 25
@@ -373,6 +373,7 @@ enum OER_FLAGS_TYPE {
 #define OER_REGISTERCONNECTION_TIMEOUT 600
 #define OER_MYSQL_CONNECTION_RETRY 60
 #define OER_MYSQL_CONNECTION_RETRY_AFTER_LOSS 1800
+#define OER_MYSQL_TIMESTAMP_CHECK_INTERVAL 300
 #define OER_BANVARS_AUTO_REJOIN 30
 #define OER_BANVARS_PART_REJOIN 600
 #define OER_BANVARS_PUBLIC_FLOOD 300
@@ -422,7 +423,7 @@ enum OER_FLAGS_TYPE {
 details possibly available on the oer+MySQL console"
 #define OER_REALNAME "http://oer.equnet.org"
 #define OER_UNKNOWN "unknown"
-#define OER_VERSION "oer+MySQL version 1.0-38"
+#define OER_VERSION "oer+MySQL version 1.0-39"
 #define OER_COPYRIGHT1 "Copyright (C) 2000-2002 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer+MySQL comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
@@ -601,6 +602,13 @@ struct channelban {
         struct channelban *prev;
 };
 
+struct syncrequest {
+	char target[CHANLEN + 1];
+	char nick[NICKLEN + 1];
+	struct syncrequest *next;
+	struct syncrequest *prev;
+};
+
 struct server {
 	char serverhost[HOSTLEN + 1];
 	char serverhost_r[HOSTLEN + 1];
@@ -668,7 +676,7 @@ struct channel {
 };
 
 struct state {
-	char config[TINYSTRINGLEN + 1];
+	char config[STRINGLEN + 1];
 	/* this is our current nick, might be temporarily set
 	to getnick when attempting to change nick */
         char nick[NICKLEN + 1];
@@ -683,7 +691,7 @@ struct state {
 	char flags[FLAGLEN + 1];
 	char ident[IDENTLEN + 1];
 	char adminsfrom[IDENTLEN + 1];
-	char usersfrom[IDENTLEN + 1];
+        char usersfrom[IDENTLEN + 1];
 	char prefix[TINYSTRINGLEN + 1];
 	char state[STRINGLEN + 1];
 	char proxysetup[STRINGLEN + 1];
@@ -715,6 +723,9 @@ struct state {
 	struct server *servers;
 	struct server *current_server;
 	struct timed *timeds;
-        struct mysqldb mysqldb;
+        struct syncrequest *syncs;
+        struct mysqldb *mysqldb;
+        struct mysqldb *mysqladmins;
+        struct mysqldb *mysqlusers;
 	struct last_action last_action;
 };
