@@ -34,6 +34,7 @@ int checkauth(struct channel *, char *, char *, int, int);
 void parseirc(char *);
 void parsecommand(char *, char *, char *, char *);
 void parsemsg(char *, char *, char *, char *);
+void parsenotice(char *, char *, char *, char *);
 
 void parseirc(char *serveroutput)
 {
@@ -287,17 +288,16 @@ void parseirc(char *serveroutput)
                                 continue;
                         }
 			ptr[0] = '\0';
+			/* reset user's flags */
+                        changeuser(this, temp_nick, -1, 0, 0, -1);
                         if(index(channel, (int)'@')) {
                                 /* @ can hide + -> set @, assume + */
                                 changeuser(this, temp_nick, -1, 1, 1, -1);
                         }
-                        if(index(channel, (int)'*')) {
-                                changeuser(this, temp_nick, 1, -1, -1, -1);
-                        }
                         if(index(channel, (int)'+')) {
                                 changeuser(this, temp_nick, -1, -1, 1, -1);
                         }
-                }
+		}
                 return;
         }
 	if(!strcasecmp(command, "330")) {
@@ -974,6 +974,16 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "NOTICE")) {
+		/* the only notice that is handled is the sync notice sent
+		   by another oer+MySQL instance in a shared setup */
+		tpos = 0;
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
+			return;
+		}
+		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
+		strncpy(channel, params, CHANLEN);
+		/* the message starts at pos + 1 ... */
+		parsenotice(temp_nick, userhost, channel, restptr + 1);
 		return;
 	}
 	if(!strcasecmp(command, "MODE")) {
@@ -1023,13 +1033,9 @@ void parseirc(char *serveroutput)
 					getnthmode(mode, i + 1, modeline);
 					if(!strcasecmp(modeline, "+k")) {
 						strncpy(this->key, target, CHANLEN);
-						snprintf(outstring, STRINGLEN, "%s channel key is now %s", this->name, this->key);
-						sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
 					}
 					if(!strcasecmp(modeline, "-k")) {
 						memset(this->key, 0, CHANLEN + 1);
-						snprintf(outstring, STRINGLEN, "deleted channel key from %s", this->name);
-						sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
 					}
 #ifdef DEBUG
 					oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->target is %s\n", target);
@@ -1406,7 +1412,6 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 	char outstring2[WRITE_BUFFER_LENGTH + 1];
 	char command[STRINGLEN + 1];
 	char channel[CHANLEN + 1];
-	char topic[OER_TOPICLEN + 1];
 	char uptimestring[STRINGLEN + 1];
 	char modeline[CHANLEN + 1];
 	char ts[TINYSTRINGLEN + 1];
@@ -1517,13 +1522,10 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                 if(!checkauth(this, userhost, nick, tochan, 0)) {
                         return;
                 }
-		if(!deltopic(this, atoi(restptr), topic)) {
+		if(!deltopic(this, (tochan) ? this->name : nick, tochan, nick, userhost, restptr)) {
 			snprintf(outstring, WRITE_BUFFER_LENGTH, "deltopic() failed");
 			sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
-			return;
 		}
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "deleted Topic #%d: %s", atoi(restptr), topic);
-		sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
 		return;
 	case OER_WHICHCOMMAND_TOPIC_EDIT:
 #ifdef DEBUG
@@ -2457,6 +2459,9 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                         return;
                 }
                 for(cb = this->channelbans; cb != NULL; cb = cb->next) {
+			if(ispermban(this, cb->ban)) {
+                                continue;
+                        }
                         mmode_new(this, mystate->now, "-b", cb->ban);
                         delchannelban(this, cb->ban);
                 }
@@ -2533,7 +2538,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			return;
 		}
 		dbclose();
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "connection to database closed");
+		snprintf(outstring, WRITE_BUFFER_LENGTH, "connection to database(s) closed");
                 sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
 		return;
 	case OER_WHICHCOMMAND_DBCONNECT:
@@ -2543,21 +2548,25 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		if(!isopa(userhost)) {
 			return;
 		}
-		if(dbping()) {
-			/* the connection is alive, do nothing */
-			return;
+		if(dbpingall()) {
+			snprintf(outstring, WRITE_BUFFER_LENGTH, "(re)connected to database(s)");
+			mystate->mysqldb->isdead = 0;
+			mystate->mysqldb->isalive = mystate->now;
+			mystate->mysqldb->tod = 0;
+			if(mystate->mysqladmins != mystate->mysqldb) {
+				mystate->mysqladmins->isdead = 0;
+				mystate->mysqladmins->isalive = mystate->now;
+				mystate->mysqladmins->tod = 0;
+			}
+			if(mystate->mysqlusers != mystate->mysqldb) {
+				mystate->mysqlusers->isdead = 0;
+				mystate->mysqlusers->isalive = mystate->now;
+				mystate->mysqlusers->tod = 0;
+			}
+		} else {
+			snprintf(outstring, WRITE_BUFFER_LENGTH, "couldn't reconnect to database(s)");
 		}
-		if(!dbconnect()) {
-			snprintf(outstring, WRITE_BUFFER_LENGTH, "couldn't reconnect to database");
-			sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
-			return;
-		}
-		if(!dbping()) {
-			return;
-		}
-		mystate->mysqldb.isalive = mystate->now;
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "reconnected to database");
-                sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
+		sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
 		return;
 	case OER_WHICHCOMMAND_RESET:
 #ifdef DEBUG
@@ -2672,3 +2681,53 @@ int checkauth(struct channel *this, char *userhost, char *nick, int tochan, int
         }
         return proceed;
 }
+
+void parsenotice(char *nick, char *userhost, char *target, char *message)
+{
+	int ppos;
+	char command[STRINGLEN + 1];
+	char p1[STRINGLEN + 1];
+	struct channel *this;
+#ifdef DEBUG
+        oer_debug(OER_DEBUG_NOISE, "parsenotice(\"%s\", \"%s\", \"%s\", \"%s\")\n", nick, userhost, target, message);
+#endif
+	if(!isme(target)) {
+		return;
+	}
+	/* parsing begins here, first parameter = the command */
+        ppos = 0;
+        if((ppos = parse(message, ppos, " ", command, STRINGLEN, 0)) < 0) {
+                return;
+        }
+	if(!strcasecmp(command, "oer_internal_sync_message")) {
+		/* admin or user sync request, sent by other oer+MySQL
+		   instances in a shared oer+MySQL setup */
+		for(this = mystate->channels; this != NULL; this = this->next) {
+			/* check for user flag "s" on any of our channels */
+			if(hasuserflags(this, userhost, "s", "")) {
+				break;
+			}
+		}
+		if(this == NULL) {
+			return;
+		}
+		if((ppos = parse(message, ppos, " ", p1, STRINGLEN, 0)) < 0) {
+			return;
+		}
+		if(isvalidchannel(p1) && ((this = getchptr(p1)) != NULL)) {
+#ifdef DEBUG
+			oer_debug(OER_DEBUG_INFO, "parsenotice->received sync request for %s from %s\n", this->name, userhost);
+#endif
+			/* sync channel */
+			syncusers(this);
+			mystate->last_ts_check = mystate->now;
+		} else if(!strcasecmp(p1, "admins")) {
+#ifdef DEBUG
+			oer_debug(OER_DEBUG_INFO, "parsenotice->received admin sync request from %s\n", userhost);
+#endif
+			/* sync admins */
+			syncadmins();
+			mystate->last_ts_check = mystate->now;
+		}
+	}
+}
